@@ -38,7 +38,7 @@ def run_forecast_ensemble(
         prediction_lag=0.0):
     # Initialize with copy of input date
     dataframe[DATE_COLUMN_NAME] = pd.to_datetime(dataframe[date_column])
-    df = dataframe.copy()
+    df = dataframe.sort_values(by=date_column)
 
     # Creates time series and ensures proper naming and frequency
     frequency = FREQUENCY_MAPPING.get(frequency)
@@ -87,8 +87,8 @@ def run_forecast_ensemble(
             train_end_date=train_end_date,
             frequency=frequency,
             forecast_period_list=period_list)
-        weight = model_dict["performance_metrics"]["r2"]
-        if weight < 0:
+        weight = model_dict["performance_metrics"]["r2"] ** 2
+        if weight < 0.5:
             weight = 0
         elif model_name == "Naive":
             weight = 0
@@ -98,40 +98,58 @@ def run_forecast_ensemble(
 
     # Combine outputs to calculate ensemble effectiveness
     print("Creating Ensemble")
-    total_weight = sum([model["weight"] for model in ensemble])
-    if total_weight == 0:
-        total_weight = 1
     return_dataframe = df.copy().set_index(DATE_COLUMN_NAME)
-    for model in ensemble:
-        # Replace negative values with the Naive value
-        forecast_df = model["forecast_dataframe"]
-        # mask = forecast_df[f"forecast_values_{model['name']}"] <= 0
-        # forecast_df.loc[mask, f"forecast_values_{model['name']}"] = estimate_df[mask, f"forecast_values_Naive"]
+    total_weight = sum([model["weight"] for model in ensemble])
+    # If no model satisfies the threshhold, Naive is the only one used
+    if total_weight == 0:
+        naive_model = next((model for model in ensemble if model['name'] == "Naive"), None)
 
-        model["weight"] = model["weight"] / total_weight
+        forecast_df = naive_model["forecast_dataframe"]
+        forecast_df[f"weighted_forecast_values_Naive"] = forecast_df["forecast_values_Naive"].copy()
         return_dataframe = pd.merge(
             left=return_dataframe,
-            right=model["forecast_dataframe"],
+            right=forecast_df,
             how="outer",
             left_index=True,
             right_index=True)
-        if model["weight"] > 0:
-            train_df = model["train_dataframe"]
-            train_df[f"weighted_predicted_values_{model['name']}"] = train_df[f"predicted_values_{model['name']}"] * model["weight"]
+
+        train_df = naive_model["train_dataframe"]
+        train_df["weighted_predicted_values_Naive"] = train_df["predicted_values_Naive"].copy()
+        return_dataframe = pd.merge(
+            left=return_dataframe,
+            right=train_df,
+            how="outer",
+            left_index=True,
+            right_index=True
+        )
+    else:
+        for model in ensemble:
+            # Replace negative values with the Naive value
+            forecast_df = model["forecast_dataframe"]
+
+            model["weight"] = model["weight"] / total_weight
             return_dataframe = pd.merge(
                 left=return_dataframe,
-                right=model["train_dataframe"],
-                how="outer",
-                left_index=True,
-                right_index=True)
-            forecast_df[f"forecast_values_{model['name']}"] = forecast_df[f"forecast_values_{model['name']}"] * model["weight"]
-            forecast_df.rename(columns={f"forecast_values_{model['name']}": f"weighted_forecast_values_{model['name']}"}, inplace=True)
-            return_dataframe = pd.merge(
-                left=return_dataframe,
-                right=forecast_df[[f"weighted_forecast_values_{model['name']}"]],
+                right=model["forecast_dataframe"],
                 how="outer",
                 left_index=True,
                 right_index=True)
+            if model["weight"] > 0:
+                train_df = model["train_dataframe"]
+                train_df[f"weighted_predicted_values_{model['name']}"] = train_df[f"predicted_values_{model['name']}"] * model["weight"]
+                return_dataframe = pd.merge(
+                    left=return_dataframe,
+                    right=train_df,
+                    how="outer",
+                    left_index=True,
+                    right_index=True)
+                forecast_df[f"weighted_forecast_values_{model['name']}"] = forecast_df[f"forecast_values_{model['name']}"] * model["weight"]
+                return_dataframe = pd.merge(
+                    left=return_dataframe,
+                    right=forecast_df[[f"weighted_forecast_values_{model['name']}"]],
+                    how="outer",
+                    left_index=True,
+                    right_index=True)
     # Create Ensemble Predictions
     ensemble_prediction_columns = [column for column in return_dataframe.columns if column.startswith("weighted_predicted_values")]
     return_dataframe["predicted_values_Ensemble"] = return_dataframe[ensemble_prediction_columns].sum(axis=1, min_count=1)
@@ -144,7 +162,8 @@ def run_forecast_ensemble(
         return_dataframe = utils.apply_lag(return_dataframe, forecast_columns, prediction_lag)
 
     # Calculate ensemble metrics
-    ensemble_train_df = return_dataframe[~return_dataframe["predicted_values_Ensemble"].isnull()]
+    required_columns = ensemble_prediction_columns + ["y"]
+    ensemble_train_df = return_dataframe.dropna(subset=required_columns)
     performance_metrics = utils.get_model_statistics(ensemble_train_df["predicted_values_Ensemble"], ensemble_train_df["y"])
     consolidated_metrics = utils.consolidate_scores(performance_metrics, ensemble_train_df["y"].mean())
 
