@@ -1,6 +1,10 @@
+import numpy as np
 import pandas as pd
-from sklearn import metrics
+
 from datetime import timedelta
+from sklearn import metrics
+from sklearn.linear_model import LinearRegression
+from statsmodels.tsa.seasonal import seasonal_decompose
 
 from amendment_forecast.models import DATE_COLUMN_NAME, VALUE_COLUMN_NAME
 # from models import DATE_COLUMN_NAME, VALUE_COLUMN_NAME
@@ -90,10 +94,15 @@ def create_backfill_time_series(original_df, frequency):
         year_mask = curve_df["year"] == (period_year + 1)
         df_compare = curve_df[period_mask & year_mask]
 
-        if len(df_compare) > 0:
+        if (len(df_compare) > 0) | (period_value == 53):
+            # If backfilling a year with 53 weeks, from a year with only 52, the 53rd week forced to be a copy
+            if (len(df_compare) == 0) & (period_value == 53):
+                period_mask = curve_df["week"] == 52
+                df_compare = curve_df[period_mask & year_mask]
+            fill_value = df_compare["y"].values[0] / annual_growth
             row = {
                 "period": missing_date,
-                "y": df_compare["y"].values[0] / annual_growth
+                "y": fill_value
             }
             rows.append(row)
 
@@ -103,3 +112,52 @@ def create_backfill_time_series(original_df, frequency):
         on="period").mean()
 
     return df_backfill
+
+
+def get_linear_predictions(train_series, prediction_index):
+
+    linear_model = LinearRegression()
+
+    x = np.array(train_series.index).reshape(-1, 1)
+    y = np.array(train_series.values).reshape(-1, 1)
+    x_future = np.array(prediction_index).reshape(-1, 1)
+
+    linear_model.fit(x, y)
+    linear_predictions = linear_model.predict(x_future)
+
+    linear_predictions = pd.Series(linear_predictions.reshape(1, -1)[0])
+
+    return linear_predictions
+
+
+def get_seasonal_decomposition(values_series):
+    # Do seasonal decomp
+    s = seasonal_decompose(values_series, model="multiplicative")
+    df_seasonal = s.seasonal
+
+    # Get full trendline
+    # TODO improve algorithm with access to internet
+    def get_first_non_null_date(series: pd.Series):
+        for index, value in series.iteritems():
+            if value == value:
+                return index
+
+    minimum_actual = get_first_non_null_date(s.trend)
+    truncated = s.trend[s.trend.index > minimum_actual]
+    full_index = truncated.index
+    truncated = truncated.reset_index(drop=True)
+
+    trend = truncated[~truncated.isnull()]
+    incomplete = truncated[truncated.isnull()]
+
+    additional_trend = get_linear_predictions(train_series=trend, prediction_index=incomplete.index)
+    full_trend = pd.concat([trend, additional_trend])
+
+    df_trend = pd.DataFrame()
+    df_trend["period_date"] = full_index.values
+    df_trend["trend"] = full_trend.reset_index(drop=True)
+    df_trend = df_trend.set_index("period_date")
+
+    df_decomposed = pd.merge(df_trend, df_seasonal, how="outer", left_index=True, right_index=True)
+
+    return df_decomposed
